#!/bin/bash -e
#
# Script to manage the start and stop of the jwt-auth-userinfo
#
SCRIPTNAME=$(basename $0)
DIRNAME=$(dirname $0)
ORIG_ARGS=("$@")
SERVICE_NAME="jwt-auth-userinfo"
cd "$DIRNAME"

function update_to_current_version {
    echo "=== Checking for jwt-auth-userinfo script updates... ==="
    TMPD=$(mktemp -d)
    TMP_ARCHIVE="${TMPD}/jwt-auth-userinfo-install-${STAGE}-${REGION}.tgz"
    # TODO: Use proper url and users
    ARCHIVE_URL="${ARCHIVE_URL:-https://artifactory.abas.sh/artifactory/abas.maven-public/jwt-auth-userinfo-install/jwt-auth-userinfo-install-${STAGE}-${REGION}.tgz}"
    wget -q -O $TMP_ARCHIVE $ARCHIVE_URL
    REMOTE_VERSION=$(tar xOf "$TMP_ARCHIVE" VERSION | head -2 | tail -1)
    LOCAL_VERSION=$(cat VERSION | head -2 | tail -1)
    if [ "$LOCAL_VERSION" -lt "$REMOTE_VERSION" ] ; then
        echo "Your installation of jwt-auth-userinfo scripts ($LOCAL_VERSION) is outdated. Version $REMOTE_VERSION is available."
        mkdir -p $PWD/backup $TMPD/backup
        mv $PWD/jwt-auth-userinfo-scripts-backup*.tgz $PWD/backup 2>/dev/null || true
        BACKUP_FILE="backup/jwt-auth-userinfo-scripts-backup-$(date -u "+%Y%m%d-%H%M%S").tgz"
        echo "Creating backup..."
        tar \
            --exclude='./jwt-auth-userinfo-scripts-backup-*.tgz' \
            --exclude='./logs' \
            --exclude='./backup' \
            -czf "$TMPD/$BACKUP_FILE" .
        mv $TMPD/$BACKUP_FILE $PWD/$BACKUP_FILE
        echo "=== Updating ==="

        tar -C "$TMPD" -xzf "$TMP_ARCHIVE"
        if [ -x "$TMPD/abas_pre_update.sh" ] ; then
            # in case we need to do something before we override the installation directory with the contents of the archive
            "$TMPD/abas_pre_update.sh" "$PWD" "$TMPD"
        fi

        mkdir -p "${PWD}/logs/state"
        local old_version="$(cat $PWD/VERSION | tr '\n' ' ')"
        local new_version="$(cat $TMPD/VERSION | tr '\n' ' ')"
        echo "{ \"message\": \"update started\", \"old_version\": \"$old_version\", \"new_version\": \"$new_version\" }" >> "${PWD}/logs/state/update.log"

        tar xzf "$TMP_ARCHIVE"
        if [ -x "$PWD/abas_post_update.sh" ] ; then
            "$PWD/abas_post_update.sh" "$PWD"
        fi

        rm -rf "$TMPD"
        echo "=== Successfully updated. ==="

        exec "$0" "${ORIG_ARGS[@]}"
    else
        echo "Skipping jwt-auth-userinfo script update. They are up-to-date."
        echo "=== Update skipped. ==="
    fi
}


function usage {
  echo "Usage: $SCRIPTNAME --up|--down [ -- additional args for docker-compose ] " 1>&2
  echo "" 1>&2
  echo "Options:" 1>&2
  echo "  --up:     Start stopped services (docker-compose up)" 1>&2
  echo "  --down:   Stop running services (docker-compose down)" 1>&2
  echo "  --status: Status of services (docker-compose ps)" 1>&2
  echo "Any arguments after the \"--\" will be passed to the docker-compose command." 1>&2
  echo "" 1>&2
  [[ $# -eq 1 ]] && exit $1 || exit 1
}

LONG_OPTS="up,down,status,help"

set +e
TEMP=$(getopt -o "$SHORT_OPTS" --long "$LONG_OPTS" -n "$SCRIPTNAME" -- "$@")
if [ $? != 0 ] ; then usage $EXIT_ERROR; fi
set -e
eval set -- "$TEMP"
while true ; do
  case "$1" in
    --up) MODE=up; shift 1 ;;
    --down) MODE=down; shift 1 ;;
    --status) MODE=status; shift 1 ;;
    --help) shift ; usage 0 ;;
    --) shift ; break ;;
    *) usage 1 ;;
  esac
done

[ -n "$MODE" ] || usage 1

# Default values, can be overwritten in $SERVICE_NAME.config
export ABAS_DOCKER_REGISTRY_NAME=docker.registry.abas.sh
export GID=$(id -g)
export UID

if test -f ./$SERVICE_NAME.config ; then
    echo "Loading configuration from $SERVICE_NAME.config"
    source ./$SERVICE_NAME.config
fi

if [ "$MODE" == "up" ]; then
  update_to_current_version
fi

mkdir -p logs/$SERVICE_NAME

DEFAULT_COMPOSE_FILE="docker-compose.yml"
COMPOSE_FILES_ARGS="-f $DEFAULT_COMPOSE_FILE"
CUSTOM_COMPOSE_FILE="docker-compose-custom.yml"
if [ -f "$CUSTOM_COMPOSE_FILE" ]; then
    COMPOSE_FILES_ARGS="$COMPOSE_FILES_ARGS -f $CUSTOM_COMPOSE_FILE"
fi

if [ "$MODE" == "up" ]; then
  docker-compose $COMPOSE_FILES_ARGS pull
  docker-compose $COMPOSE_FILES_ARGS up -d --remove-orphans "$@"
elif [ "$MODE" == "down" ]; then
  docker-compose $COMPOSE_FILES_ARGS down "$@"
elif [ "$MODE" == "status" ]; then
  docker-compose $COMPOSE_FILES_ARGS ps "$@"
fi
